/*
 *   @(#) $Id: ProtocolSessionFilterChain.java 165302 2005-04-29 12:53:46Z trustin $
 *
 *   Copyright 2004 The Apache Software Foundation
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */
package org.apache.mina.protocol;

import org.apache.mina.io.IoFilterChain;
import org.apache.mina.io.IoSession;
import org.apache.mina.io.IoSessionManagerFilterChain;

/**
 * An {@link IoFilterChain} that forwards <tt>filterWrite</tt>
 * requests to the specified {@link IoSessionManagerFilterChain}.
 * <p>
 * This filter chain is used by implementations of {@link IoSession}s.
 * 
 * @author The Apache Directory Project (dev@directory.apache.org)
 * @author Trustin Lee (trustin@apache.org)
 * @version $Rev: 165302 $, $Date: 2005-04-29 21:53:46 +0900 $
 */
public class ProtocolSessionFilterChain extends AbstractProtocolFilterChain {

    private final ProtocolSessionManagerFilterChain managerChain;

    public ProtocolSessionFilterChain( ProtocolSessionManagerFilterChain managerChain )
    {
        this.managerChain = managerChain;
    }

    protected void doWrite( ProtocolSession session, Object message )
    {
        managerChain.filterWrite( session, message );
    }
}
